#include "mem_utils.h"
#include "mem_utils_ops.h"
#include "aliases.h"
#include "platform.h"
#include <assert.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#if defined(WAPP_PLATFORM_WINDOWS)
#include "mem_utils_win.h"
#elif defined(WAPP_PLATFORM_POSIX)
#include "mem_utils_posix.h"
#else
#error "Unrecognised platform"
#endif

internal bool is_power_of_two(u64 num) { return (num & (num - 1)) == 0; }

void *wapp_mem_util_align_forward(void *ptr, u64 alignment) {
  if (!ptr) {
    return NULL;
  }

  assert(is_power_of_two(alignment));

  uptr p = (uptr)ptr;
  uptr align = (uptr)alignment;

  // Similar to p % align, but it's a faster implementation that works fine
  // because align is guaranteed to be a power of 2
  uptr modulo = p & (align - 1);

  if (modulo != 0) {
    p += align - modulo;
  }

  return (void *)p;
}

void *wapp_mem_util_alloc(void *addr, u64 size, MemAccess access, MemAllocFlags flags, MemInitType type) {
  void *output = mem_util_allocate(addr, size, access, flags, type);

  if (type == WAPP_MEM_INIT_INITIALISED) {
    memset(output, 0, size);
  }

  return output;
}

void wapp_mem_util_free(void *ptr, u64 size) {
  mem_util_free(ptr, size);
}
